import pyarrow as pa #import pyarrow library
import pyarrow.parquet as pq
import pandas as pd #import pandas library
import os

# Specify the full path to the directory containing the Parquet files
input_dir = r"C:\Users\Nurbulat\Desktop\MDP\yellow"

# Creating  list of all parquet files in the directory
parquet_files = [os.path.join(input_dir, f) for f in os.listdir(input_dir) if f.endswith('.parquet')]

for file in parquet_files:
    df = pq.read_table(file).to_pandas()

# processing data loop with dropping rows
dfs = []  # listing to store cleaned DataFrames
for file in parquet_files:
    df = pq.read_table(file).to_pandas()
        # capturing the original row count
    original_row_count = len(df)
    # Additional data cleaning and filtering specified in the Data Processing chapter
    df = df[(df['PULocationID'] != "") &
        (df['DOLocationID'] != "") &
        (df['tpep_pickup_datetime'].notna()) & 
        (df['tpep_dropoff_datetime'].notna()) & 
        ((df['VendorID'] == 1) | (df['VendorID'] == 2)) &
        ((df['RatecodeID'] == 1) | (df['RatecodeID'] == 2) | (df['RatecodeID'] == 3) | (df['RatecodeID'] == 4) | (df['RatecodeID'] == 5) | (df['RatecodeID'] == 6)) &  
        ((df['store_and_fwd_flag'] == "Y") | (df['store_and_fwd_flag'] == "N")) &
        (df['passenger_count'] >= 1) & 
        (df['passenger_count'] <= 4) & 
        (df['fare_amount'] > 0) & 
        (df['fare_amount'] < 1000) & 
        (df['trip_distance'] > 0) & 
        (df['trip_distance'] < 100) & 
        (df['total_amount'] > 0) & 
        ((df['payment_type'] == 1) | (df['payment_type'] == 2)) & 
        (df['congestion_surcharge'].notna()) & 
        (df['airport_fee'].notna()) &
        (df['mta_tax'].notna()) & 
        (df['tip_amount'].notna()) & 
        (df['extra'].notna())]

    # drop unnecessary columns
    df = df.drop(['RatecodeID', 'store_and_fwd_flag', 'improvement_surcharge'], axis=1)
    
   # Filter out rows with no transition IDs
    unique_pu = set(df['PULocationID'].unique())
    unique_do = set(df['DOLocationID'].unique())
    no_transition_pu_ids = unique_pu - unique_do
    no_transition_do_ids = unique_do - unique_pu
    df = df[~df['PULocationID'].isin(no_transition_pu_ids)]
    df = df[~df['DOLocationID'].isin(no_transition_do_ids)]

    # Capturing the filtered row count
    filtered_row_count = len(df)
    #printing the original row cound and the filtered 
    print(f"File: {file}")
    print(f"Original Row Count: {original_row_count}")
    print(f"Filtered Row Count: {filtered_row_count}")

# Extract only the filename and add 'cleaned_' to it
    cleaned_filename = 'cleaned_' + os.path.basename(file)
    cleaned_file_path = os.path.join(input_dir, cleaned_filename)
    pq.write_table(pa.Table.from_pandas(df), cleaned_file_path)
    dfs.append(df)

# concatenate all cleaned dataframes
clean_df = pd.concat(dfs, ignore_index=True)

# setting display options to show all columns
pd.set_option('display.max_columns', None)

# Display cleaned dataframe with all columns
print(clean_df)

# storing cleaned dataframe to a new parque file
pq.write_table(pa.Table.from_pandas(clean_df), os.path.join(input_dir, 'formatted_yellow_taxi_22.parquet'))

# calculate median, mean, and standard deviation of fare_amount column
fare_median = clean_df['fare_amount'].median()
fare_mean = clean_df['fare_amount'].mean()
fare_std = clean_df['fare_amount'].std()

# analyze additional columns
# analyze most common and least common pickup hours
pickup_counts = clean_df.groupby(clean_df['tpep_pickup_datetime'].dt.hour)['tpep_pickup_datetime'].count()
max_pickup_hour = pickup_counts.idxmax()
min_pickup_hour = pickup_counts.idxmin()

# analyze the most and the least common passengers count
passenger_counts = clean_df['passenger_count'].value_counts()
most_common_passenger_count = passenger_counts.idxmax()
rare_passenger_count = passenger_counts.idxmin()

# analyze the most popular and the least popular pickup and dropoff locations
pickup_counts = clean_df['PULocationID'].value_counts()
popular_pickup_locations = pickup_counts.idxmax()
unpopular_pickup_locations = pickup_counts.idxmin()
dropoff_counts = clean_df['DOLocationID'].value_counts()
popular_dropoff_locations = dropoff_counts.idxmax()
unpopular_dropoff_locations = dropoff_counts.idxmin()

# calculate median, mean, and standard deviation of trip_distance column
distance_median = clean_df['trip_distance'].median()
distance_mean = clean_df['trip_distance'].mean()
distance_std = clean_df['trip_distance'].std()

fare_by_location = clean_df.groupby(['PULocationID', 'DOLocationID'])['fare_amount'].mean()
max_fare_location = fare_by_location.idxmax()
min_fare_location = fare_by_location.idxmin()

# display the cleaned DataFrame with all columns
pd.set_option('display.max_columns', None)
print(clean_df)

# display the median, mean, and standard deviation of fare_amount
print('Fare Median:', fare_median)
print('Fare Mean:', fare_mean)
print('Fare Standard Deviation:', fare_std)

# display the analysis of additional columns
print('Peak pickup hour:', max_pickup_hour)
print('Off-peak pickup hour:', min_pickup_hour)
print('Most common passenger count:', most_common_passenger_count)
print('Most rare passenger count:', rare_passenger_count)
print('Distance Median:', distance_median)
print('Distance Mean:', distance_mean)
print('Popular pickup locations:', popular_pickup_locations)
print('Unpopular pickup locations:', unpopular_pickup_locations)
print('Popular dropoff locations:', popular_dropoff_locations )
print('Unpopular dropoff locations:', unpopular_dropoff_locations)
print('Distance Standard Deviation:', distance_std)
print('Location with highest average fare:', max_fare_location)
print('Location with lowest average fare:', min_fare_location)
