#import libraries
import numpy as np
import pandas as pd

#define the path
file_path = 'yellow/formatted_yellow_taxi_22.parquet'
taxi_data = pd.read_parquet(file_path)

def calculate_rewards_optimized(taxi_data):
     # Constants for airport locations and fee
    JFK_AIRPORT_ID = 132
    LAGUARDIA_AIRPORT_ID = 138
    AIRPORT_FEE = 1.25

    # Convert timestamps to datetime if not already
    taxi_data['pickup_datetime'] = pd.to_datetime(taxi_data['tpep_pickup_datetime'])
    taxi_data['dropoff_datetime'] = pd.to_datetime(taxi_data['tpep_dropoff_datetime'])

    # Calculate trip duration in seconds
    taxi_data['trip_duration'] = (taxi_data['dropoff_datetime'] - taxi_data['pickup_datetime']).dt.total_seconds()

    # Unique location IDs from both PULocationID and DOLocationID
    all_locations = np.union1d(taxi_data['PULocationID'].unique(), taxi_data['DOLocationID'].unique())
    all_locations.sort()  # Sort for consistent indexing

    # Map location IDs to indices in the matrix
    location_to_index = {loc: idx for idx, loc in enumerate(all_locations)}

    # Initialize the reward matrix
    num_states = len(all_locations)
    reward_matrix = np.zeros((num_states, num_states))

    # Calculate net rewards including the trip duration
    taxi_data['net_reward'] = taxi_data['total_amount'] - taxi_data['extra'] - taxi_data['mta_tax'] - taxi_data['congestion_surcharge']

    # Adjust for airport fee
    taxi_data.loc[taxi_data['DOLocationID'].isin([JFK_AIRPORT_ID, LAGUARDIA_AIRPORT_ID]), 'net_reward'] -= AIRPORT_FEE
    
     # Small number to avoid division by zero
    epsilon = 1e-6
    taxi_data['adjusted_reward'] = taxi_data['net_reward'] * taxi_data['trip_distance'] / (taxi_data['trip_duration'] + epsilon)
 
    # Aggregate rewards for each state-action pair
    aggregated_rewards = taxi_data.groupby(['PULocationID', 'DOLocationID'])['adjusted_reward'].sum().reset_index()

    # Populate the reward matrix
    for _, row in aggregated_rewards.iterrows():
        pu_index = location_to_index[row['PULocationID']]
        do_index = location_to_index[row['DOLocationID']]
        reward_matrix[pu_index, do_index] = row['adjusted_reward']

    # Normalize the reward matrix
    reward_matrix /= taxi_data.shape[0]

    assert np.all(np.isfinite(reward_matrix)), "Reward matrix contains non-finite values"


    return reward_matrix


reward_matrix = calculate_rewards_optimized(taxi_data)

# printing  reward matrix
print(reward_matrix)