import pyarrow.parquet as pq
import pandas as pd

# set the file path for the Parquet file
file_path = 'formatted_yellow_taxi_22.parquet'

# open the file using PyArrow
table = pq.read_table(file_path)

# convert the PyArrow table to a Pandas DataFrame
df = table.to_pandas()

# calculate median, mean, and standard deviation of fare_amount column
fare_median = df['fare_amount'].median()
fare_mean = df['fare_amount'].mean()
fare_std = df['fare_amount'].std()

# analyze additional columns
pickup_counts = df.groupby(df['tpep_pickup_datetime'].dt.hour)['tpep_pickup_datetime'].count()
max_pickup_hour = pickup_counts.idxmax()
min_pickup_hour = pickup_counts.idxmin()

passenger_counts = df['passenger_count'].value_counts()
most_common_passenger_count = passenger_counts.idxmax()

distance_median = df['trip_distance'].median()
distance_mean = df['trip_distance'].mean()
distance_std = df['trip_distance'].std()

fare_by_location = df.groupby(['PULocationID', 'DOLocationID'])['fare_amount'].mean()
max_fare_location = fare_by_location.idxmax()
min_fare_location = fare_by_location.idxmin()

tip_by_location = df.groupby(['PULocationID', 'DOLocationID'])['tip_amount'].mean()
max_tip_location = tip_by_location.idxmax()
min_tip_location = tip_by_location.idxmin()

popular_pickup_location = df['PULocationID'].value_counts()
max_pickup_location = popular_pickup_location.idxmax()
min_pickup_location = popular_pickup_location.idxmin()

popular_dropoff_location = df['DOLocationID'].value_counts()
max_dropoff_location = popular_dropoff_location.idxmax()
min_dropoff_location = popular_dropoff_location.idxmin()



# display the cleaned DataFrame with all columns
pd.set_option('display.max_columns', None)
print(df)

# display the median, mean, and standard deviation of fare_amount
print('Fare Median:', fare_median)
print('Fare Mean:', fare_mean)
print('Fare Standard Deviation:', fare_std)

# display the analysis of additional columns
print('Peak pickup hour:', max_pickup_hour)
print('Off-peak pickup hour:', min_pickup_hour)
print('Most common passenger count:', most_common_passenger_count)
print('Distance Median:', distance_median)
print('Distance Mean:', distance_mean)
print('Distance Standard Deviation:', distance_std)
print('Location with highest average fare:', max_fare_location)
print('Location with lowest average fare:', min_fare_location)
print('Location with highest average tip:', max_tip_location)
print('Location with lowest average tip:', min_tip_location)
print('Popular pickup location:', max_pickup_location)
print('Unpopular pickup location:', min_pickup_location)
print('Popular dropoff location:', max_dropoff_location)
print('Unpopular dropoff location:', min_dropoff_location)

